/**********************************************************************************
 * The Configurator allows to easily edit the TS2PerlMod config.
 * Copyright (C) 2006  Felix Siefert
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 **********************************************************************************/

#include <QGridLayout>
#include <QLabel>
#include <QGroupBox>
#include <QPixmap>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QDebug>
#include <QVariant>
#include <QIntValidator>
#include <QMessageBox>
#include <QCheckBox>
#include <QLineEdit>

#include "common.h"
#include "settings.h"
#include "features.h"

Common::Common(QWidget *parent) : QWidget(parent)
{
    m_description  = new QLabel(tr("The Configurator is a GUI application written in C++ "
                                   "that allows you to add or edit the configuration files "
                                   "of the TS2PerlMod.\n\n"
                                   "Please visit the Planet TeamSpeak forums if you have any "
                                   "bugs to report."));
    m_description->setWordWrap(true);
    m_version      = new QLabel(tr("Configurator 0.6 RC3"));
    m_version->setAlignment(Qt::AlignRight | Qt::AlignBottom);
    m_generalGroup = new QGroupBox(tr("General"));
    m_connectionGroup = new QGroupBox(tr("Connection"));
    m_loginGroup      = new QGroupBox(tr("Login"));
    m_logo            = new QLabel;
    m_logo->setPixmap(QPixmap(":/logo.png"));
    m_logo->setAlignment(Qt::AlignHCenter);
    
    QValidator *intValidator = new QIntValidator(0, 65535, this);
    
    QGridLayout *mainLayout = new QGridLayout;
    mainLayout->addWidget(m_logo, 0, 0, 1, 0);
    mainLayout->addWidget(m_description, 1, 0, 1, 0);
    mainLayout->addWidget(m_generalGroup, 2, 0, 1, 0);
    mainLayout->addWidget(m_connectionGroup, 3, 0, 2, 1);
    mainLayout->addWidget(m_loginGroup, 3, 1,2, 1);
    //mainLayout->addWidget(m_version, 4, 1);
    setLayout(mainLayout);
    
    /* General Group **************************************/
    m_generalInterval = new TimeInput(tr("Repeat Timer:"), 76, true);
    m_generalLogging = new QCheckBox(tr("Logfile:"));
    m_generalLogfile = new QLineEdit();
    m_generalInterval->setTimeSec(2);
    m_generalLogging->setMaximumWidth(76);
    m_generalLogging->setMinimumWidth(76);
    m_generalLogfile->setEnabled(false);
    m_generalLogfile->setText("default.log");
    connect(m_generalLogging, SIGNAL(toggled(bool)),
            this, SLOT(toggleLogEdit(bool)));
    
    QHBoxLayout *logLayout = new QHBoxLayout;
    logLayout->addWidget(m_generalLogging);
    logLayout->addWidget(m_generalLogfile);
    
    QVBoxLayout *generalLayout = new QVBoxLayout;
    generalLayout->addWidget(m_generalInterval);
    generalLayout->addLayout(logLayout);
    m_generalGroup->setLayout(generalLayout);
    
    /* Connection Group ***********************************/
    m_serverAddress = new InputWidget(tr("Server Address:"), 80);
    m_serverTcp = new InputWidget(tr("Server TCP Port:"), 80);
    m_serverTcp->setValidator(intValidator);
    m_serverUdp = new InputWidget(tr("Server UDP Port:"), 80);
    m_serverUdp->setValidator(intValidator);
    QVBoxLayout *connectionLayout = new QVBoxLayout;
    connectionLayout->addWidget(m_serverAddress);
    connectionLayout->addWidget(m_serverTcp);
    connectionLayout->addWidget(m_serverUdp);
    m_connectionGroup->setLayout(connectionLayout);
    
    /* Login Group ****************************************/
    m_loginDescription = new QLabel(tr("This is the username and password you use to access your TeamSpeak 2 "
                                       "server. This must be an existing superadmin account."));
    m_loginDescription->setWordWrap(true);
    m_loginUser = new InputWidget(tr("Username:"), 76);
    m_loginPassLabel = new QLabel(tr("Password:"));
    m_loginPass = new QLineEdit;
    m_loginShow = new QCheckBox(tr("Show"));
    m_loginPassLabel->setMaximumWidth(76);
    m_loginPassLabel->setMinimumWidth(76);
    m_loginPass->setEchoMode(QLineEdit::Password);
    connect(m_loginShow, SIGNAL(toggled(bool)),
            this, SLOT(toggleEchoMode(bool)));
    
    QHBoxLayout *passLayout = new QHBoxLayout;
    passLayout->addWidget(m_loginPassLabel);
    passLayout->addWidget(m_loginPass);
    passLayout->addWidget(m_loginShow);
    
    QVBoxLayout *loginLayout = new QVBoxLayout;
    loginLayout->addWidget(m_loginDescription);
    loginLayout->addWidget(m_loginUser);
    loginLayout->addLayout(passLayout);
    m_loginGroup->setLayout(loginLayout);
}

void Common::toggleLogEdit(bool checked)
{
    if(checked)
    {
        m_generalLogfile->setEnabled(true);
    }
    else
    {
        m_generalLogfile->setEnabled(false);
    }
}

void Common::toggleEchoMode(bool checked)
{
    if(checked)
    {
        m_loginPass->setEchoMode(QLineEdit::Normal);
    }
    else
    {
        m_loginPass->setEchoMode(QLineEdit::Password);
    }
}

void Common::load(QSettings &settings)
{
    m_serverAddress->setText(settings.value("CONNECTION/ServerAddress").toString());
    m_serverTcp->setText(settings.value("CONNECTION/ServerTCPPort").toString());
    m_serverUdp->setText(settings.value("CONNECTION/ServerUDPPort").toString());
    
    m_loginUser->setText(settings.value("LOGIN/Username").toString());
    m_loginPass->setText(settings.value("LOGIN/Password").toString());
    
    m_generalInterval->setTimeSec(settings.value("TIMER/RepeatTimer").toInt());
    m_generalLogging->setChecked(settings.value("LOG/Enabled").toBool());
    m_generalLogfile->setText(settings.value("LOG/File").toString());
}

void Common::save(QSettings &settings)
{
    QString address = m_serverAddress->getText();
    QString tcp     = m_serverTcp->getText();
    QString udp     = m_serverUdp->getText();
    QString user    = m_loginUser->getText();
    QString pass    = m_loginPass->text();
    int     timer   =  m_generalInterval->getTime();
    
    if(address.isEmpty() || tcp.isEmpty() || udp.isEmpty()
    || user.isEmpty() || pass.isEmpty())
    {
        QMessageBox::information(this, "Configurator",
        tr("Note that not all required fields are filled in."));
    }
    if(timer < 2)
    {
        QMessageBox::information(this, "Configurator",
        tr("Repeat Timer can't be lower than 2 seconds."));
    }
    
    settings.setValue("CONNECTION/ServerAddress", address);
    settings.setValue("CONNECTION/ServerTCPPort", tcp);
    settings.setValue("CONNECTION/ServerUDPPort", udp);
    
    settings.setValue("LOGIN/Username", user);
    settings.setValue("LOGIN/Password", pass);
    
    settings.setValue("TIMER/RepeatTimer", timer);
    settings.setValue("LOG/Enabled", (int) m_generalLogging->isChecked());
    settings.setValue("LOG/File", m_generalLogfile->text());
}
