/**********************************************************************************
 * The Configurator allows to easily edit the TS2PerlMod config.
 * Copyright (C) 2006  Felix Siefert
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 **********************************************************************************/

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTextEdit>
#include <QPushButton>
#include <QLabel>
#include <QInputDialog>
#include <QFileDialog>
#include <QFile>
#include <QFileInfo>
#include <QTextStream>
#include <QMessageBox>
#include <QMenu>

#include "editor.h"

Highlighter::Highlighter(QStringList &keywords, QTextDocument *parent) : QSyntaxHighlighter(parent)
{
    HighlightingRule rule;

    keywordFormat.setForeground(Qt::darkRed);
    keywordFormat.setFontWeight(QFont::Bold);
    
    for(int i = 0; i < keywords.size(); ++i) {
        QString pattern = keywords.at(i);
        rule.pattern = QRegExp(pattern);
        rule.format = keywordFormat;
        highlightingRules.append(rule);
    }
}

void Highlighter::highlightBlock(const QString &text)
{
    for(int i = 0; i < highlightingRules.size(); ++i) {
        HighlightingRule rule = highlightingRules.at(i);
        QRegExp expression(rule.pattern);
        int index = text.indexOf(expression);
        while (index >= 0) {
            int length = expression.matchedLength();
            setFormat(index, length, rule.format);
            index = text.indexOf(expression, index + length);
        }
    }
}

Editor::Editor(bool commands, QWidget *parent) : QWidget(parent)
{
    if(commands)
    {
        m_player_vars << "<PLAYER_ID>"
                      << "<PLAYER_NICKNAME>"
                      << "<PLAYER_LOGINNAME>"
                      << "<PLAYER_PACKETS_SENT>"
                      << "<PLAYER_PACKETS_RECEIVED>"
                      << "<PLAYER_DATA_SENT>"
                      << "<PLAYER_DATA_RECEIVED>"
                      << "<PLAYER_PACKET_LOSS>"
                      << "<PLAYER_PING>"
                      << "<PLAYER_IP>"
                      << "<PLAYER_LOGINTIME>"
                      << "<PLAYER_IDLETIME>"
                      << "<PLAYER_FLAGS>";
        m_channel_vars << "<CHANNEL_ID>"
                       << "<CHANNEL_NAME>"
                       << "<CHANNEL_FLAGS>"
                       << "<CHANNEL_CODEC>"
                       << "<CHANNEL_ORDER>"
                       << "<CHANNEL_MAX_USERS>"
                       << "<CHANNEL_TOPIC>";
        m_server_vars << "<SERVER_ID>"
                      << "<SERVER_NAME>"
                      << "<SERVER_PORT>"
                      << "<SERVER_UPTIME>"
                      << "<SERVER_VERSION>"
                      << "<SERVER_LICENSE>"
                      << "<SERVER_PLATFORM>"
                      << "<SERVER_CURRENT_USERS>"
                      << "<SERVER_MAX_USERS>"
                      << "<SERVER_CURRENT_CHANNELS>"
                      << "<SERVER_PACKETS_SENT>"
                      << "<SERVER_PACKETS_RECEIVED>"
                      << "<SERVER_DATA_SENT>"
                      << "<SERVER_DATA_RECEIVED>"
                      << "<SERVER_BANDWIDTH_IN>"
                      << "<SERVER_BANDWIDTH_OUT>"
                      << "<SERVER_BANDWIDTH_TOTAL>"
                      << "<SERVER_WEBPOST_POSTURL>"
                      << "<SERVER_WEBPOST_LINKURL>";
        m_all_vars = m_player_vars + m_channel_vars + m_server_vars;
    }
    
    m_text        = new QTextEdit;
    m_text->setFontFamily("MS Sans Serif");
    m_highlighter = new Highlighter(m_all_vars, m_text->document());
    m_commandsBtn = new QPushButton(tr("Variables"));
    m_loadBtn     = new QPushButton(tr("Load"));
    m_nameBtn     = new QPushButton(tr("Change Name"));
    m_file        = new QLabel(tr("<no filename set>"));
    m_file->setAlignment(Qt::AlignRight);
    m_commandsMenu = new QMenu(tr("Commands"), this);
    m_player_commandsMenu = new QMenu(tr("Player Commands"), this);
    m_channel_commandsMenu = new QMenu(tr("Channel Commands"), this);
    m_server_commandsMenu = new QMenu(tr("Server Commands"), this);
    
    setupCommandsMenu();
    
    connect(m_commandsMenu, SIGNAL(triggered(QAction*)), this, SLOT(onMenuAction(QAction*)));
    connect(m_loadBtn, SIGNAL(clicked()), this, SLOT(showLoadDialog()));
    connect(m_nameBtn, SIGNAL(clicked()), this, SLOT(showNameDialog()));
    
    /* buttonsLayout */
    QHBoxLayout *buttonsLayout = new QHBoxLayout;
    if(commands)
    {
        buttonsLayout->addWidget(m_commandsBtn);
    }
    buttonsLayout->addStretch();
    buttonsLayout->addWidget(m_loadBtn);
    buttonsLayout->addWidget(m_nameBtn);
    
    /* mainLayout */
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setMargin(0);
    mainLayout->addWidget(m_file);
    mainLayout->addLayout(buttonsLayout);
    mainLayout->addWidget(m_text);
    setLayout(mainLayout);
}

void Editor::setupCommandsMenu()
{
    for(int i = 0; i < m_player_vars.size(); ++i) {
        QString var = m_player_vars.at(i);
        m_player_commandsMenu->addAction(var);
    }
    
    for(int i = 0; i < m_channel_vars.size(); ++i) {
        QString var = m_channel_vars.at(i);
        m_channel_commandsMenu->addAction(var);
    }
    
    for(int i = 0; i < m_server_vars.size(); ++i) {
        QString var = m_server_vars.at(i);
        m_server_commandsMenu->addAction(var);
    }
    
    m_commandsMenu->addMenu(m_player_commandsMenu);
    m_commandsMenu->addMenu(m_channel_commandsMenu);
    m_commandsMenu->addMenu(m_server_commandsMenu);
    m_commandsBtn->setMenu(m_commandsMenu);
}

void Editor::setFile(const QString &file)
{
    m_file->setText(file);
}

QString Editor::getFile() const
{
    return m_file->text();
}

void Editor::setText(const QString &text)
{
    m_text->setPlainText(text);
}

QString Editor::getText() const
{
    return m_text->toPlainText();
}

void Editor::onMenuAction(QAction *action)
{
    QString keyword = action->text();
    m_text->insertPlainText(keyword);
}

void Editor::showLoadDialog()
{
    QString textDocs = tr("Textdocuments");
    
    QString path = QFileDialog::getOpenFileName(
                   this,
                   tr("Choose a file to open"),
                   "text",
                   textDocs + " (*.txt)");
    
    if(path.isEmpty())
    {
        return;
    }
    
    QFile file(path);
    
    if(!file.open(QIODevice::ReadOnly))
    {
        QMessageBox::information(this, "Configurator",
            tr("Unable to open the file.\n"),
            path);
        return;
    }
    
    QTextStream stream(&file);
    m_text->setPlainText(stream.readAll());
    //emit textChanged();
    
    QFileInfo info(path);
    m_file->setText(info.fileName());
    //emit fileChanged();
}

void Editor::showNameDialog()
{
    bool ok;
    QString name = QInputDialog::getText(this, tr("Set File Name"),
                                             tr("File:"), QLineEdit::Normal,
                                             m_file->text(), &ok);
    if(ok && !name.isEmpty())
    {
        QRegExp rx("*.txt");
        rx.setPatternSyntax(QRegExp::Wildcard);
        if(!rx.exactMatch(name))
        {
            name.append(".txt");
        }
        m_file->setText(name);
        //emit fileChanged();
    }
}
