/**********************************************************************************
 * The Configurator allows to easily edit the TS2PerlMod config.
 * Copyright (C) 2006  Felix Siefert
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 **********************************************************************************/

#include <QDebug>
#include <QGridLayout>
#include <QVBoxLayout>
#include <QComboBox>
#include <QVariant>
#include <QLabel>
#include <QLineEdit>
#include <QGroupBox>
#include <QCheckBox>
#include <QSpinBox>
#include <QPushButton>
#include <QMenu>
#include <QAction>

#include "features.h"

InputWidget::InputWidget(const QString &label, int width, QWidget *parent) : QWidget(parent)
{
    m_label = new QLabel(label);
    m_edit  = new QLineEdit;
    
    if(width > 0)
    {
        m_label->setMaximumWidth(width);
        m_label->setMinimumWidth(width);
    }
    
    QHBoxLayout *mainLayout = new QHBoxLayout;
    mainLayout->setMargin(0);
    mainLayout->addWidget(m_label);
    mainLayout->addWidget(m_edit);
    setLayout(mainLayout);
}

QString InputWidget::getText() const
{
    return m_edit->text();
}

void InputWidget::setText(const QString &text)
{
    m_edit->setText(text);
}

void InputWidget::setValidator(const QValidator *v)
{
    m_edit->setValidator(v);
}

TimeInput::TimeInput(const QString &label, int width, bool sec, QWidget *parent) : QWidget(parent)
{
    m_label = new QLabel(label);
    m_spinBox = new QSpinBox;
    m_comboBox = new QComboBox;
    
    m_spinBox->setMaximum(9999);
    
    if(width > 0)
    {
        m_label->setMaximumWidth(width);
        m_label->setMinimumWidth(width);
    }
    
    if(sec)
    {
        m_comboBox->addItem(tr("seconds"));
    }
    m_comboBox->addItem(tr("minutes"));
    m_comboBox->addItem(tr("hours"));
    
    QHBoxLayout *mainLayout = new QHBoxLayout;
    mainLayout->setMargin(0);
    mainLayout->addWidget(m_label);
    mainLayout->addWidget(m_spinBox);
    mainLayout->addWidget(m_comboBox);
    setLayout(mainLayout);
}

int TimeInput::getTime() const
{
    int time;
    
    QString currentText = m_comboBox->currentText();
    if(currentText == tr("minutes"))
    {
        time = m_spinBox->value() * 60;
    }
    else if(currentText == tr("hours"))
    {
        time = m_spinBox->value() * 60 * 60;
    }
    else
    {
        time = m_spinBox->value();
    }
    
    return time;
}

int TimeInput::getTimeMin() const
{
    int time;
    
    QString currentText = m_comboBox->currentText();
    if(currentText == tr("minutes"))
    {
        time = m_spinBox->value();
    }
    else if(currentText == tr("hours"))
    {
        time = m_spinBox->value() * 60;
    }
    else
    {
        time = m_spinBox->value() / 60;
    }
    
    return time;
}

void TimeInput::setTimeSec(int sec)
{
    if(!(sec % (60 * 60)))
    {
        m_spinBox->setValue(sec / (60 * 60));
        int index = m_comboBox->findText(tr("hours"));
        m_comboBox->setCurrentIndex(index);
    }
    else if(!(sec % 60))
    {
        m_spinBox->setValue(sec / 60);
        int index = m_comboBox->findText(tr("minutes"));
        m_comboBox->setCurrentIndex(index);
    }
    else
    {
        m_spinBox->setValue(sec);
        int index = m_comboBox->findText(tr("seconds"));
        m_comboBox->setCurrentIndex(index);
    }
}

void TimeInput::setTimeMin(int min)
{
    if(!(min % 60))
    {
        m_spinBox->setValue(min / 60);
        int index = m_comboBox->findText(tr("hours"));
        m_comboBox->setCurrentIndex(index);
    }
    else
    {
        m_spinBox->setValue(min);
        int index = m_comboBox->findText(tr("minutes"));
        m_comboBox->setCurrentIndex(index);
    }
}

Punishment::Punishment(QWidget *parent) : QWidget(parent)
{
    m_punishLabel = new QLabel(tr("Punishment:"));
    m_comboBox    = new QComboBox;
    m_label       = new QLabel(tr("Message:"));
    m_lineEdit    = new QLineEdit;
    m_channelLabel = new QLabel(tr("Channel:"));
    m_channel     = new QLineEdit;
    
    m_comboBox->addItem("move", QVariant(Settings::MOVE));
    m_comboBox->addItem("kick", QVariant(Settings::KICK));
    m_comboBox->addItem("remove", QVariant(Settings::REMOVE));
    m_comboBox->addItem("ban", QVariant(Settings::BAN));
    
    connect(m_comboBox, SIGNAL(currentIndexChanged(const QString&)),
            this, SLOT(onIndexChanged(const QString&)));
    
    QGridLayout *mainLayout = new QGridLayout;
    mainLayout->setMargin(0);
    mainLayout->addWidget(m_punishLabel, 0, 0);
    mainLayout->addWidget(m_comboBox, 0, 1);
    mainLayout->addWidget(m_label, 1, 0);
    mainLayout->addWidget(m_lineEdit, 1, 1);
    mainLayout->addWidget(m_channelLabel, 2, 0);
    mainLayout->addWidget(m_channel, 2, 1);
    setLayout(mainLayout);
}

void Punishment::onIndexChanged(const QString &text)
{
    if(text == QString("move"))
    {
        m_channel->setEnabled(true);
        m_label->setText(tr("Message:"));
    }
    else if(text == QString("kick"))
    {
        m_channel->setEnabled(false);
        m_label->setText(tr("Reason:"));
    }
    else if(text == QString("remove"))
    {
        m_channel->setEnabled(false);
        m_label->setText(tr("Message:"));
    }
    else if(text == QString("ban"))
    {
        m_channel->setEnabled(false);
        m_label->setText(tr("Reason:"));
    }
}

void Punishment::setPunish(Settings::Pain pain)
{
    int index = m_comboBox->findData(QVariant(pain));
    m_comboBox->setCurrentIndex(index);
}

void Punishment::setLabel(const QString &label)
{
    m_lineEdit->setText(label);
}

void Punishment::setChannel(const QString &label)
{
    m_channel->setText(label);
}

Settings::Pain Punishment::getPunish() const
{
    int index = m_comboBox->currentIndex();
    return static_cast<Settings::Pain>(m_comboBox->itemData(index).toInt());
}

QString Punishment::getPunishStr() const
{
    QString str;
    Settings::Pain pain = getPunish();
    switch(pain)
    {
        case Settings::MOVE: str = "move"; break;
        case Settings::KICK: str = "kick"; break;
        case Settings::REMOVE: str = "remove"; break;
        case Settings::BAN: str = "ban"; break;
        default: str = "kick";
    }
    return str;
}

QString Punishment::getLabel() const
{
    return m_lineEdit->text();
}

QString Punishment::getChannel() const
{
    return m_channel->text();
}

Features::Features(QWidget *parent) : QWidget(parent)
{
    m_advancedGroup  = new QGroupBox(tr("Advanced"));
    m_moveidlerGroup = new QGroupBox(tr("Move Idler to Channel"));
    m_warnidlerGroup = new QGroupBox(tr("Warn Idler"));
    m_kickidlerGroup = new QGroupBox(tr("Kick Idler"));
    m_autoafkGroup   = new QGroupBox(tr("Auto AFK"));
    m_antirecGroup   = new QGroupBox(tr("Anti Recording"));
    m_stickchannelGroup   = new QGroupBox(tr("Sticky Channel"));
    m_antifloodGroup      = new QGroupBox(tr("Anti Flood"));
    m_takeoverGroup       = new QGroupBox(tr("Take Over Protection"));
	m_autoservertypeGroup = new QGroupBox(tr("Auto Servertype"));
	m_updatebansGroup     = new QGroupBox(tr("Update Banlist"));
	m_bugdetectionGroup   = new QGroupBox(tr("Bug Detection"));
    
    m_moveidlerGroup->setCheckable(true);
    m_warnidlerGroup->setCheckable(true);
    m_kickidlerGroup->setCheckable(true);
    m_autoafkGroup->setCheckable(true);
    m_antirecGroup->setCheckable(true);
    m_stickchannelGroup->setCheckable(true);
    m_antifloodGroup->setCheckable(true);
    m_takeoverGroup->setCheckable(true);
	m_autoservertypeGroup->setCheckable(true);
	m_updatebansGroup->setCheckable(true);
	m_bugdetectionGroup->setCheckable(true);
    
    m_moveidlerGroup->setChecked(false);
    m_warnidlerGroup->setChecked(false);
    m_kickidlerGroup->setChecked(false);
    m_autoafkGroup->setChecked(false);
    m_antirecGroup->setChecked(false);
    m_stickchannelGroup->setChecked(false);
    m_antifloodGroup->setChecked(false);
    m_takeoverGroup->setChecked(false);
	m_autoservertypeGroup->setChecked(false);
	m_updatebansGroup->setChecked(false);
	m_bugdetectionGroup->setChecked(false);
    
    /* advanced *******************************************/
    m_extendedmsg = new QCheckBox(tr("Extended Welcome Message"));
    m_badnick     = new QCheckBox(tr("Bad Nicknames"));
	m_badchannel  = new QCheckBox(tr("Bad Channels"));
    m_advert      = new QCheckBox(tr("Advert Message"));
    m_stats       = new QCheckBox(tr("Stats Message"));
    m_protuser    = new QCheckBox(tr("Protected Users"));
    
    QVBoxLayout *advancedLayout = new QVBoxLayout;
    advancedLayout->addWidget(m_extendedmsg);
    advancedLayout->addWidget(m_badnick);
	advancedLayout->addWidget(m_badchannel);
    advancedLayout->addWidget(m_advert);
    advancedLayout->addWidget(m_stats);
    advancedLayout->addWidget(m_protuser);
    m_advancedGroup->setLayout(advancedLayout);
    
    /* Anti Rec *******************************************/
    m_antirecPunish = new Punishment;
    QVBoxLayout *antirecLayout = new QVBoxLayout;
    antirecLayout->addWidget(m_antirecPunish);
    m_antirecGroup->setLayout(antirecLayout);
    
    /* Move Idler *****************************************/
    m_moveidlerTime = new TimeInput(tr("Idletime:"), 46);
    m_moveidlerInput = new InputWidget(tr("Channel:"), 46);
    m_moveidlerTime->setTimeMin(60);
    QVBoxLayout *moveidlerLayout = new QVBoxLayout;
    moveidlerLayout->addWidget(m_moveidlerTime);
    moveidlerLayout->addWidget(m_moveidlerInput);
    m_moveidlerGroup->setLayout(moveidlerLayout);
    
    /* Warn Idler *****************************************/
    m_warnidlerTime = new TimeInput(tr("Idletime"), 46);
    m_warnidlerInput = new InputWidget(tr("Message:"), 46);
    m_warnidlerTime->setTimeMin(90);
    QVBoxLayout *warnidlerLayout = new QVBoxLayout;
    warnidlerLayout->addWidget(m_warnidlerTime);
    warnidlerLayout->addWidget(m_warnidlerInput);
    m_warnidlerGroup->setLayout(warnidlerLayout);
    
    /* Kick Idler *****************************************/
    m_kickidlerTargetBtn = new QPushButton(tr("Targets"));
    QMenu *kiMenu = new QMenu();
    m_kiPlayerR = new QAction(tr("Target Player R"), this);
    m_kiPlayerU = new QAction(tr("Target Player U"), this);
    m_kiChannelR = new QAction(tr("Target Channel R"), this);
    m_kiChannelU = new QAction(tr("Target Channel U"), this);
    
    m_kiPlayerR->setCheckable(true);
    m_kiPlayerU->setCheckable(true);
    m_kiChannelR->setCheckable(true);
    m_kiChannelU->setCheckable(true);
    
    kiMenu->addAction(m_kiPlayerR);
    kiMenu->addAction(m_kiPlayerU);
    kiMenu->addAction(m_kiChannelR);
    kiMenu->addAction(m_kiChannelU);
    m_kickidlerTargetBtn->setMenu(kiMenu);
    
    m_kickidlerTime = new TimeInput(tr("Idletime:"), 46);
    m_kickidlerInput = new InputWidget(tr("Reason:"), 46);
    m_kickidlerTime->setTimeMin(120);
    QVBoxLayout *kickidlerLayout = new QVBoxLayout;
    kickidlerLayout->addWidget(m_kickidlerTargetBtn);
    kickidlerLayout->addWidget(m_kickidlerTime);
    kickidlerLayout->addWidget(m_kickidlerInput);
    m_kickidlerGroup->setLayout(kickidlerLayout);
    
    /* Auto Afk *******************************************/
    m_autoafkInput = new InputWidget(tr("Channel:"), 46);
    QVBoxLayout *autoafkLayout = new QVBoxLayout;
    autoafkLayout->addWidget(m_autoafkInput);
    m_autoafkGroup->setLayout(autoafkLayout);
    
    /* Stick Channel **************************************/
    m_stickchannelInput = new InputWidget(tr("Channel:"), 46);
    QVBoxLayout *stickchannelLayout = new QVBoxLayout;
    stickchannelLayout->addWidget(m_stickchannelInput);
    m_stickchannelGroup->setLayout(stickchannelLayout);
    
    /* Anti Flood *****************************************/
    m_antifloodMax = new InputWidget(tr("Max Connections:"), 90);
    m_antifloodSec = new InputWidget(tr("Max Seconds:"), 90);
    QVBoxLayout *antifloodLayout = new QVBoxLayout;
    antifloodLayout->addWidget(m_antifloodMax);
    antifloodLayout->addWidget(m_antifloodSec);
    m_antifloodGroup->setLayout(antifloodLayout);
    
    /* Take Over ******************************************/
    m_takeoverPunish = new Punishment;
    QVBoxLayout *takeoverLayout = new QVBoxLayout;
    takeoverLayout->addWidget(m_takeoverPunish);
    m_takeoverGroup->setLayout(takeoverLayout);

	/* Auto Servertype ************************************/
	m_autotypeClients  = new InputWidget(tr("Clients:"), 46);
	m_autotypeInterval = new InputWidget(tr("Interval:"), 46);
	QVBoxLayout *autotypeLayout = new QVBoxLayout;
	autotypeLayout->addWidget(m_autotypeClients);
	autotypeLayout->addWidget(m_autotypeInterval);
	m_autoservertypeGroup->setLayout(autotypeLayout);

	/* Update Bans ****************************************/
	m_updatebansTime = new TimeInput(tr("Bantime"), 46);
	QVBoxLayout *updatebansLayout = new QVBoxLayout;
	updatebansLayout->addWidget(m_updatebansTime);
	m_updatebansGroup->setLayout(updatebansLayout);

	/* Bug Detection **************************************/
	m_bugdetectionLabel = new QLabel(tr("No options available"));
	QVBoxLayout *bugdetectionLayout = new QVBoxLayout;
	bugdetectionLayout->addWidget(m_bugdetectionLabel);
	m_bugdetectionGroup->setLayout(bugdetectionLayout);
    
    /* mainLayout *****************************************/
    QGridLayout *mainLayout = new QGridLayout;
    /*mainLayout->addWidget(m_advancedGroup, 0, 0, 2, 1);
    mainLayout->addWidget(m_moveidlerGroup, 2, 0, 2, 1);
    mainLayout->addWidget(m_warnidlerGroup, 4, 0, 2, 1);
    mainLayout->addWidget(m_kickidlerGroup, 0, 1);
    mainLayout->addWidget(m_autoafkGroup, 1, 1, 2, 1);
    mainLayout->addWidget(m_antirecGroup, 3, 1, 2, 1);
    mainLayout->addWidget(m_stickchannelGroup, 5, 1);
    mainLayout->addWidget(m_antifloodGroup, 6, 0);
    mainLayout->addWidget(m_takeoverGroup, 6, 1);*/
    
    mainLayout->addWidget(m_advancedGroup, 0, 0, 2, 1);
    mainLayout->addWidget(m_moveidlerGroup, 2, 0, 2, 1);
    mainLayout->addWidget(m_warnidlerGroup, 4, 0, 2, 1);
    mainLayout->addWidget(m_kickidlerGroup, 6, 0, 2, 1);
    mainLayout->addWidget(m_antirecGroup, 0, 1, 1, 1);
    mainLayout->addWidget(m_antifloodGroup, 1, 1, 2, 1);
    mainLayout->addWidget(m_takeoverGroup, 3, 1, 3, 1);
    mainLayout->addWidget(m_stickchannelGroup, 6, 1, 1, 1);
    mainLayout->addWidget(m_autoafkGroup, 7, 1, 1, 1);
	mainLayout->addWidget(m_autoservertypeGroup, 8, 0, 2, 1);
	mainLayout->addWidget(m_updatebansGroup, 8, 1, 1, 1);
	mainLayout->addWidget(m_bugdetectionGroup, 9, 1, 1, 1);
    
    setLayout(mainLayout);
}

void Features::load(QSettings &settings)
{
    m_extendedmsg->setChecked(settings.value("EXTENDED-WELCOME-MESSAGE/Enabled").toBool());
    m_badnick->setChecked(settings.value("BAD-NICKNAME/Enabled").toBool());
	m_badchannel->setChecked(settings.value("BAD-CHANNEL/Enabled").toBool());
    m_advert->setChecked(settings.value("ADVERT-MESSAGE/Enabled").toBool());
    m_stats->setChecked(settings.value("STATS-MESSAGE/Enabled").toBool());
    m_protuser->setChecked(settings.value("PROTECTED-USERS/Enabled").toBool());
    
    m_moveidlerGroup->setChecked(settings.value("MOVE-IDLERS/Enabled").toBool());
    m_moveidlerTime->setTimeMin(settings.value("MOVE-IDLERS/Idletime").toInt());
    m_moveidlerInput->setText(settings.value("MOVE-IDLERS/Channel").toString());
    
    m_warnidlerGroup->setChecked(settings.value("WARN-IDLERS/Enabled").toBool());
    m_warnidlerTime->setTimeMin(settings.value("WARN-IDLERS/Idletime").toInt());
    m_warnidlerInput->setText(settings.value("WARN-IDLERS/Message").toString());
    
    m_kickidlerGroup->setChecked(settings.value("KICK-IDLERS/Enabled").toBool());
    m_kickidlerTime->setTimeMin(settings.value("KICK-IDLERS/Idletime").toInt());
    m_kiPlayerR->setChecked(settings.value("KICK-IDLERS/TargetPlayerR").toBool());
    m_kiPlayerU->setChecked(settings.value("KICK-IDLERS/TargetPlayerU").toBool());
    m_kiChannelR->setChecked(settings.value("KICK-IDLERS/TargetChannelR").toBool());
    m_kiChannelU->setChecked(settings.value("KICK-IDLERS/TargetChannelU").toBool());
    m_kickidlerInput->setText(settings.value("KICK-IDLERS/Reason").toString());
    
    Settings::Pain antirecPain = stringToPain(settings.value("ANTI-REC/Punishment").toString());
    m_antirecGroup->setChecked(settings.value("ANTI-REC/Enabled").toBool());
    m_antirecPunish->setPunish(antirecPain);
    m_antirecPunish->setChannel(settings.value("ANTI-REC/Channel").toString());
    if(antirecPain == Settings::MOVE || antirecPain == Settings::REMOVE)
    {
        m_antirecPunish->setLabel(settings.value("ANTI-REC/Message").toString());
    }
    else
    {
        m_antirecPunish->setLabel(settings.value("ANTI-REC/Reason").toString());
    }
    
    m_antifloodGroup->setChecked(settings.value("ANTI-FLOOD/Enabled").toBool());
    m_antifloodMax->setText(settings.value("ANTI-FLOOD/MaxConnections").toString());
    m_antifloodSec->setText(settings.value("ANTI-FLOOD/MaxSeconds").toString());

    Settings::Pain takeoverPain = stringToPain(settings.value("TAKE-OVER-PROTECTION/Punishment").toString());
    m_takeoverGroup->setChecked(settings.value("TAKE-OVER-PROTECTION/Enabled").toBool());
    m_takeoverPunish->setPunish(takeoverPain);
    m_takeoverPunish->setChannel(settings.value("TAKE-OVER-PROTECTION/Channel").toString());
    if(takeoverPain == Settings::MOVE || takeoverPain == Settings::REMOVE)
    {
        m_takeoverPunish->setLabel(settings.value("TAKE-OVER-PROTECTION/Message").toString());
    }
    else
    {
        m_takeoverPunish->setLabel(settings.value("TAKE-OVER-PROTECTION/Reason").toString());
    }
    
    m_stickchannelGroup->setChecked(settings.value("STICKY-CHANNEL/Enabled").toBool());
    m_stickchannelInput->setText(settings.value("STICKY-CHANNEL/Channel").toString());
    
    m_autoafkGroup->setChecked(settings.value("AUTO-AFK/Enabled").toBool());
    m_autoafkInput->setText(settings.value("AUTO-AFK/Channel").toString());

	m_autoservertypeGroup->setChecked(settings.value("AUTO-SERVERTYPE/Enabled").toBool());
	m_autotypeClients->setText(settings.value("AUTO-SERVERTYPE/Clients").toString());
	m_autotypeInterval->setText(settings.value("AUTO-SERVERTYPE/Interval").toString());

	m_updatebansGroup->setChecked(settings.value("UPDATE-BANLIST/Enabled").toBool());
	m_updatebansTime->setTimeMin(settings.value("UPDATE-BANLIST/Bantime").toInt());

	m_bugdetectionGroup->setChecked(settings.value("BUG-DETECTION/Enabled").toBool());
}

void Features::save(QSettings &settings)
{
    settings.setValue("EXTENDED-WELCOME-MESSAGE/Enabled", (int) m_extendedmsg->isChecked());
    settings.setValue("BAD-NICKNAME/Enabled", (int) m_badnick->isChecked());
	settings.setValue("BAD-CHANNEL/Enabled", (int) m_badchannel->isChecked());
    settings.setValue("ADVERT-MESSAGE/Enabled", (int) m_advert->isChecked());
    settings.setValue("STATS-MESSAGE/Enabled", (int) m_stats->isChecked());
    settings.setValue("PROTECTED-USERS/Enabled", (int) m_protuser->isChecked());
    
    settings.setValue("MOVE-IDLERS/Enabled", (int) m_moveidlerGroup->isChecked());
    settings.setValue("MOVE-IDLERS/Idletime", m_moveidlerTime->getTimeMin());
    settings.setValue("MOVE-IDLERS/Channel", m_moveidlerInput->getText());
    
    settings.setValue("WARN-IDLERS/Enabled", (int) m_warnidlerGroup->isChecked());
    settings.setValue("WARN-IDLERS/Idletime", m_warnidlerTime->getTimeMin());
    settings.setValue("WARN-IDLERS/Message", m_warnidlerInput->getText());
    
    settings.setValue("KICK-IDLERS/Enabled", (int) m_kickidlerGroup->isChecked());
    settings.setValue("KICK-IDLERS/Idletime", m_kickidlerTime->getTimeMin());
    settings.setValue("KICK-IDLERS/TargetPlayerR", (int) m_kiPlayerR->isChecked());
    settings.setValue("KICK-IDLERS/TargetPlayerU", (int) m_kiPlayerU->isChecked());
    settings.setValue("KICK-IDLERS/TargetChannelR", (int) m_kiChannelR->isChecked());
    settings.setValue("KICK-IDLERS/TargetChannelU", (int) m_kiChannelU->isChecked());
    settings.setValue("KICK-IDLERS/Reason", m_kickidlerInput->getText());
    
    settings.setValue("ANTI-REC/Enabled", (int) m_antirecGroup->isChecked());
    settings.setValue("ANTI-REC/Punishment", m_antirecPunish->getPunishStr());
    settings.setValue("ANTI-REC/Channel", m_antirecPunish->getChannel());
    if(m_antirecPunish->getPunish() == Settings::MOVE || m_antirecPunish->getPunish() == Settings::REMOVE)
    {
        settings.setValue("ANTI-REC/Message", m_antirecPunish->getLabel());
    }
    else
    {
        settings.setValue("ANTI-REC/Reason", m_antirecPunish->getLabel());
    }
    
    settings.setValue("ANTI-FLOOD/Enabled", (int) m_antifloodGroup->isChecked());
    settings.setValue("ANTI-FLOOD/MaxConnections", m_antifloodMax->getText());
    settings.setValue("ANTI-FLOOD/MaxSeconds", m_antifloodSec->getText());
    
    settings.setValue("TAKE-OVER-PROTECTION/Enabled", (int) m_takeoverGroup->isChecked());
    settings.setValue("TAKE-OVER-PROTECTION/Punishment", m_takeoverPunish->getPunishStr());
    settings.setValue("TAKE-OVER-PROTECTION/Channel", m_takeoverPunish->getChannel());
    if(m_antirecPunish->getPunish() == Settings::MOVE || m_takeoverPunish->getPunish() == Settings::REMOVE)
    {
        settings.setValue("TAKE-OVER-PROTECTION/Message", m_takeoverPunish->getLabel());
    }
    else
    {
        settings.setValue("TAKE-OVER-PROTECTION/Reason", m_takeoverPunish->getLabel());
    }
    
    settings.setValue("STICKY-CHANNEL/Enabled", (int) m_stickchannelGroup->isChecked());
    settings.setValue("STICKY-CHANNEL/Channel", m_stickchannelInput->getText());
    
    settings.setValue("AUTO-AFK/Enabled", (int) m_autoafkGroup->isChecked());
    settings.setValue("AUTO-AFK/Channel", m_autoafkInput->getText());

	settings.setValue("AUTO-SERVERTYPE/Enabled", (int) m_autoservertypeGroup->isChecked());
	settings.setValue("AUTO-SERVERTYPE/Clients", m_autotypeClients->getText());
	settings.setValue("AUTO-SERVERTYPE/Interval", m_autotypeInterval->getText());

	settings.setValue("UPDATE-BANLIST/Enabled", (int) m_updatebansGroup->isChecked());
	settings.setValue("UPDATE-BANLIST/Bantime", m_updatebansTime->getTimeMin());

	settings.setValue("BUG-DETECTION/Enabled", (int) m_bugdetectionGroup->isChecked());
}

bool Features::cstb(Qt::CheckState state)
{
    if(state != Qt::Unchecked)
    {
       return true;
    }
    else
    {
        return false;
    }
}
